@@ -1,17 +1,34 @@
 # -*- coding: utf-8 -*-
-# shared classes
+# shared infrastructure
 
 import numpy as np                                                # type: ignore
-import scipy                                                      # type: ignore
+import scipy as sp                                                # type: ignore
 import warnings
 from abc import ABC, abstractmethod
+from functools import wraps
 from numpy import ndarray
 from typing import Callable, Optional, Sequence, Tuple, Type, Union, Iterable
 from .util import check_X_update, check_X_y, check_X_y_update, \
                   is_array, map_array, concat_values, concat_bounds, V, B
 
+__all__ = ['Thetas']
 
-OPT_BACKENDS = ('scipy')
+OPT_BACKENDS = {'scipy': sp.optimize.minimize}              # optimizer backends
+SPL_BACKENDS = {'numpy': np.random}                           # sampler backends
+
+
+def verify_density_operands(density_operator):
+    """ decorator for overloading density operators """
+    @wraps(density_operator)
+    def wrapped_density_operator(self, operand):
+        if isinstance(operand, Density):
+            if self.n_variates == operand.n_variates:
+                return density_operator(self, operand)
+            else:
+                return ValueError('densities must agree on n_variates')
+        else:
+            raise ValueError('a density operator only accepts two densities.')
+    return wrapped_density_operator
 
 
 class Density(ABC):
@@ -22,13 +39,18 @@ class Density(ABC):
         """ initialize density object """
 
     @abstractmethod
-    def __call__(self, x):
+    def __call__(self, x: ndarray):
         """ evaluate density at x """
 
     @abstractmethod
     def symmetric(self) -> bool:
         """ symmetric or asymmetric """
 
+    @property
+    @abstractmethod
+    def n_variates(self) -> int:
+        """ number of variates i.e. len(x) """
+
     def _log_ratio(self, x_star, x):
         """ q(x|x_star) / q(x_star|x) """
         if self.symmetric():
@@ -36,11 +58,56 @@ class Density(ABC):
         else:
             raise NotImplementedError
 
+    @verify_density_operands
+    def __mul__(self, other):
+        if isinstance(self, Distribution) and isinstance(other, Distribution):
+            return ProductDistribution(self, other)
+        else:
+            return ProductDensity(self, other)
+
+    @verify_density_operands
+    def __rmul__(self, other):
+        if isinstance(other, Distribution) and isinstance(self, Distribution):
+            return ProductDistribution(other, self)
+        else:
+            return ProductDensity(other, self)
+
+
+class ProductDensity(Density):
+    """ product density """
+
+    def __init__(self, d1: Density, d2: Density):
+        self._d1 = d1
+        self._d2 = d2
+
+    def __call__(self, x: ndarray):
+        return self.d1(x) *  self.d2(x)
+
+    @property
+    def d1(self):
+        return self._d1
+
+    @property
+    def d2(self):
+        return self._d2
+
+    def symmetric(self) -> bool:
+        """ sufficient but not necessary condition for symmetry """
+        return self.d1.symmetric() and self.d2.symmetric()
+
+    @property
+    def n_variates(self):
+        return self.d1.n_variates
+
 
 class Distribution(Density):
     """ normalized probability density """
 
 
+class ProductDistribution(Distribution, ProductDensity):
+    """ product distribution """
+
+
 class Bounds:
     """
     Thetas' component
@@ -82,28 +149,25 @@ class Bounds:
                        np.concatenate([other.uppers, self.uppers]) )
 
     @property
-    def lowers(self):
+    def lowers(self) -> ndarray:
+        """ lower bound """
         return self._lowers
 
     @property
-    def uppers(self):
+    def uppers(self) -> ndarray:
+        """ upper bound """
         return self._uppers
 
     def contains(self, values: ndarray) -> bool:
+        """ returns true if values are bounded within [lower, upper] """
         if values.shape != self.lowers.shape:
             raise ValueError('dimension of values must agree with bounds.')
         return np.all(self.lowers - 1e-8 <= values) and \
                np.all(self.uppers + 1e-8 >= values)
 
-    def clamped(self) -> bool:  # FIXME: use fixed or not to replace it
-        if np.allclose(self.lowers, self.uppers):
-            return True
-        else:
-            return False
-
     def get(self, backend: str = 'scipy'):
         if backend == 'scipy':
-            return scipy.optimize.Bounds(self.lowers, self.uppers)
+            return sp.optimize.Bounds(self.lowers, self.uppers)
         else:
             raise ValueError('backend must be one of {}'.format(OPT_BACKENDS))
 
@@ -151,18 +215,22 @@ class Thetas:
     @classmethod
     def from_seq(cls, values: Sequence[V], bounds: Sequence[B],
                  transform: Callable = lambda x: x):
+        """ a convenience method to construct thetas with sequence of values """
         return Thetas(values=map_array(transform, concat_values(*values)),
                       bounds=Bounds.from_seq(bounds, transform=transform))
 
     @property
-    def values(self):
+    def values(self) -> ndarray:
+        """ values """
         return self._values
 
     @property
-    def bounds(self):
+    def bounds(self) -> Bounds:
+        """ bounds """
         return self._bounds
 
     def set(self, values: ndarray):
+        """ set values and check whether bounds are violated """
         if not is_array(values, 1, np.number):
             raise TypeError('values must be 1d numeric array.')
         if not self.bounds.contains(values):
@@ -176,13 +244,16 @@ class Thetas:
         """
         parameters are assigned with values/distribution or not
         special case: zero-size theta returns True
+        .. todo:: returns True if thetas has None as values but has prior dst
         """
-        # FIXME: returns True if thetas has None as values but has prior dst
         return np.all(np.isfinite(self.values))
 
 
 class Hypers:
-    """ a view of learnable and fixed parameters """
+    """
+    a view of learnable and fixed parameters
+    .. todo:: potentially no longer useful
+    """
 
     def __init__(self, names: ndarray, values: ndarray):
         if isinstance(names, (tuple, list)):
@@ -235,7 +306,7 @@ class Model(ABC):
 
     @property
     @abstractmethod
-    def thetas(self):
+    def thetas(self) -> Thetas:
         """ core parametrisation of the model """
 
     def parametrised(self) -> bool:
@@ -261,7 +332,7 @@ class UnsupervisedModel(Model):
         self.X = X
 
     @abstractmethod
-    def transform(self, X: ndarray):
+    def transform(self, X: ndarray) -> ndarray:
         if not self.fitted():
             raise AttributeError('model is not fitted yet.')
         check_X_update(self.X, X)
@@ -288,7 +359,7 @@ class SupervisedModel(Model):
         self.y =y
 
     @abstractmethod
-    def predict(self, X: ndarray):
+    def predict(self, X: ndarray) -> ndarray:
         if not self.parametrised():
             raise AttributeError('model not parametrised yet.')
         if self.fitted():
