@@ -130,7 +130,7 @@ class GaussianProcessRegressor(BayesianSupervisedModel):
             mu, sigma = p.mu.item(), sqrt(p.cov.item())
             y_min = self.y.min()
             return (y_min - mu) * norm.cdf(y_min, mu, sigma) + \
-                   sigma**2 * norm.pdf(y_min, mu, sigma)
+                   sigma * norm.pdf(y_min, mu, sigma)
         # lower confidence bound
         def fun_lcb(x: ndarray, beta: float = 1.) -> float:
             assert is_array(x, 1, np.number)
@@ -169,24 +169,23 @@ class GaussianProcessRegressor(BayesianSupervisedModel):
             return
         else:
             return
-        # only support uninformative prior θ for now
 
     def hyper_posterior(self, n_samples: int = 0, **kwargs):
         super().hyper_posterior(n_samples)
-        hyper_posterior = LogDensity(partial(self._obj(self.X, self.y),
-                                             grad=False)               )
+        log_mll = partial(self._obj(self.X, self.y), grad=False)
+        k = len(self.thetas)
+        hyper_posterior = LogDensity(log_mll, k)
         if n_samples <= 0:
             return hyper_posterior
         else:
-            k = len(self.thetas)
             sampler = MarkovChainMonteCarloSampler(hyper_posterior,
                           Gaussian(np.zeros(k), np.eye(k)),
-                          self.thetas.values, **kwargs)
-            return sampler.sample(n_samples)
-        # only support uninformative prior θ for now
+                          self.thetas.values, n_samples, **kwargs)
+            return sampler.sample()
+        # only support flat hyperprior θ for now
 
     def fit(self, X: ndarray, y: ndarray, verbose: bool = False):
-        """ MAP estimatem under uniform prior """
+        """ empirical bayes || type II maximum likelihood """
         super().fit(X, y)
         self.optimizer.fun = self._obj(self.X, self.y)
         self.optimizer.jac = True
@@ -223,7 +222,7 @@ class GaussianProcessRegressor(BayesianSupervisedModel):
             return prior
         else:
             return prior.sample(n_samples)
-        # only support uninformative prior θ for now
+        # only support Dirac θ for now
 
     def posterior_predictive(self, X: ndarray, n_samples: int = 0) \
         -> Union[Gaussian, ndarray]:
@@ -232,16 +231,16 @@ class GaussianProcessRegressor(BayesianSupervisedModel):
         Kzz = self.kernel(X, X)
         mu = np.einsum('ij,j->i', Kzx, self.dual_weights)
         cov = Kzz - Kzx @ cho_solve((self.L, True), Kzx.T)
-        if np.any(cov < 0.):
-            warnings.warn('posterior covariance matrix has negative elements. '
-                          'possibly numerical issues. correcting to 0.')
-        cov[cov < 0.] = 0.
+        # if np.any(cov < 0.):
+        #     warnings.warn('posterior covariance has negative elements. '
+        #                   'possibly numerical issues - correcting to 0.')
+        # cov[cov < 0.] = 0.
         posterior = Gaussian(mu, cov, allow_singular=True)
         if n_samples <= 0:
             return posterior
         else:
             return posterior.sample(n_samples)
-        # only support uninformative prior θ for now
+        # only support Dirac θ for now
 
 
 class tProcessRegressor(BayesianSupervisedModel):
