@@ -6,25 +6,25 @@ from fractions import Fraction
 from functools import wraps
 import numpy as np                                                # type: ignore
 from math import exp, log, sqrt, pi
-from numpy import ndarray, newaxis
+from numpy import ndarray, newaxis as nax
 from typing import Any, Callable, Optional, Sequence, Tuple, Type, Union
 from ..base import Model, Bounds, Thetas, Hypers
 from ..metric import dist
 from ..util import audit_X, audit_X_Z, B, V, is_array, concat_bounds
 
 
-def check_operand(operator):
+def verify_kernel_operands(kernel_operator):
     """ decorator for overloading kernel operators """
-    @wraps(operator)
-    def wrapped_operator(self, operand):
+    @wraps(kernel_operator)
+    def wrapped_kernel_operator(self, operand):
         if isinstance(operand, Kernel):
-            return operator(self, operand)
+            return kernel_operator(self, operand)
         elif isinstance(operand, float) and operand > 0.:
-            return operator(self, ConstantKernel(operand))
+            return kernel_operator(self, ConstantKernel(operand))
         else:
             raise ValueError('a kernel operator accepts either two kernels '
                              'or a positive float and a kernel as operands.')
-    return wrapped_operator
+    return wrapped_kernel_operator
 
 
 class Kernel(Model):
@@ -32,7 +32,7 @@ class Kernel(Model):
 
     @property
     @abstractmethod
-    def thetas(self):
+    def thetas(self) -> Thetas:
         """
         learnable hyperparameters and their bounds
         used for model internals
@@ -40,7 +40,7 @@ class Kernel(Model):
 
     @property
     @abstractmethod
-    def hyperparameters(self):
+    def hyperparameters(self) -> Hypers:
         """
         fixed and learnable hyperparameters (view of thetas)
         used for printing model specifications
@@ -68,19 +68,19 @@ class Kernel(Model):
         compute kernel K(X, Z)
         """
 
-    @check_operand
+    @verify_kernel_operands
     def __add__(self, other):
         return Sum(self, other)
 
-    @check_operand
+    @verify_kernel_operands
     def __radd__(self, other):
         return Sum(other, self)
 
-    @check_operand
+    @verify_kernel_operands
     def __mul__(self, other):
         return Product(self, other)
 
-    @check_operand
+    @verify_kernel_operands
     def __rmul__(self, other):
         return Product(other, self)
 
@@ -91,19 +91,19 @@ class Kernel(Model):
             raise ValueError('kernel exponentiation only accepts '
                              'an integer or a float as exponent')
 
-    @check_operand
+    @verify_kernel_operands
     def __or__(self, other):
         return KroneckerSum(self, other)
 
-    @check_operand
+    @verify_kernel_operands
     def __ror__(self, other):
         return KroneckerSum(other, self)
 
-    @check_operand
+    @verify_kernel_operands
     def __and__(self, other):
         return KroneckerProduct(self, other)
 
-    @check_operand
+    @verify_kernel_operands
     def __rand__(self, other):
         return KroneckerProduct(other, self)
 
@@ -127,23 +127,23 @@ class Sum(Kernel):
         return '{k1} + {k2}'.format(k1=self.k1, k2=self.k2)
 
     @property
-    def k1(self):
+    def k1(self) -> Kernel:
         return self._k1
 
     @property
-    def k2(self):
+    def k2(self) -> Kernel:
         return self._k2
 
     @property
-    def b(self):
+    def b(self) -> int:
         return self._b
 
     @property
-    def thetas(self):
+    def thetas(self) -> Thetas:
         return self.k1.thetas + self.k2.thetas
 
     @property
-    def hyperparameters(self):
+    def hyperparameters(self) -> Hypers:
         return self.k1.hyperparameters + self.k2.hyperparameters
 
     def stationary(self) -> bool:
@@ -153,7 +153,7 @@ class Sum(Kernel):
         self.k1._set(log_params[:self.b])
         self.k2._set(log_params[self.b:])
 
-    def _obj(self, X: ndarray):
+    def _obj(self, X: ndarray) -> Callable:
         super()._obj(X)
         f1 = self.k1._obj(X)
         f2 = self.k2._obj(X)
@@ -475,7 +475,7 @@ class ConstantKernel(StationaryMixin, Kernel):
             assert is_array(log_params, 1, np.number)
             a = exp(log_params[0])
             K = U * a
-            d_K_d_loga = K[:, :, newaxis]
+            d_K_d_loga = K[:, :, nax]
             return K, d_K_d_loga
         return fun
 
@@ -590,7 +590,7 @@ class RBFKernel(StationaryMixin, Kernel):
                 # kernel
                 K = np.exp(R2_l2 / -2.)
                 # jacobian
-                d_K_d_logl = (K * R2_l2)[:, :, newaxis]
+                d_K_d_logl = (K * R2_l2)[:, :, nax]
                 return K, d_K_d_logl
         else:
             """ check dimensions """
@@ -603,14 +603,14 @@ class RBFKernel(StationaryMixin, Kernel):
                 # length scale
                 l = np.exp(log_params)
                 X_l = np.einsum('ij,j->ij', X, 1./l)
-                # pairwise scaled difference squared (feature being 3rd axis)
-                d_K_d_logl = (X_l[:, newaxis, :] - X_l[newaxis, :, :])**2
+                # pairwise scaled difference squared (z axis: feature)
+                d_K_d_logl = (X_l[:, nax, :] - X_l[nax, :, :])**2
                 # pairwise scaled l2 distance squared (summing over feature)
                 R2_l2 = d_K_d_logl.sum(axis=2)
                 # kernel
                 K = np.exp(R2_l2 / -2.)
                 # jacobian
-                d_K_d_logl *= K[:, :, newaxis]
+                d_K_d_logl *= K[:, :, nax]
                 return K, d_K_d_logl
         return fun
 
@@ -644,7 +644,7 @@ class RationalQuadraticKernel(StationaryMixin, Kernel):
     m (mixture coefficient): shape parameter of gamma distribution over
                              RBF's inverse squared length scales l**2 (rate
                              parameter is set to be proportioal to l**2);
-                             converges to RBF kernel as m goes to infinity
+                             tends to RBF kernel as m -> inf
     l (average length scale): positive float
     """
 
@@ -724,8 +724,8 @@ class RationalQuadraticKernel(StationaryMixin, Kernel):
                 m = exp(log_params[0])
                 l = np.exp(log_params[1:])
                 X_l = np.einsum('ij,j->ij', X, 1./l)
-                # pairwise scaled difference squared (feature being 3rd axis)
-                d_K_d_logl = (X_l[:, newaxis, :] - X_l[newaxis, :, :])**2
+                # pairwise scaled difference squared (z axis: feature)
+                d_K_d_logl = (X_l[:, nax, :] - X_l[nax, :, :])**2
                 # pairwise scaled distance squared (summing over feature)
                 R2_l2 = d_K_d_logl.sum(axis=2)
                 # kernel
@@ -733,7 +733,7 @@ class RationalQuadraticKernel(StationaryMixin, Kernel):
                 K = B**-m
                 # jacobian
                 d_K_d_logm = m * K * (1. - 1./B - np.log(B))
-                d_K_d_logl *= (K / B)[:, :, newaxis]
+                d_K_d_logl *= (K / B)[:, :, nax]
                 return K, np.dstack((d_K_d_logm, d_K_d_logl))
         return fun
 
@@ -850,9 +850,9 @@ class MaternKernel(StationaryMixin, Kernel):
                 K = J * f(B)
                 # jacobian
                 if d == 1:
-                    d_K_d_logl = (J * B)[:, :, newaxis]  # g(B) = 1
+                    d_K_d_logl = (J * B)[:, :, nax]  # g(B) = 1
                 else:
-                    d_K_d_logl = (J * B * g(B))[:, :, newaxis]
+                    d_K_d_logl = (J * B * g(B))[:, :, nax]
                 return K, d_K_d_logl
         else:
             def fun(log_params: ndarray) -> Tuple[ndarray, ndarray]:
@@ -860,8 +860,8 @@ class MaternKernel(StationaryMixin, Kernel):
                 # length scale
                 l = np.exp(log_params)
                 X_l = np.einsum('ij,j->ij', X, 1./l)
-                # pairwise scaled difference squared (feature being 3rd axis)
-                d_K_d_logl = (X_l[:, newaxis, :] - X_l[newaxis, :, :])**2
+                # pairwise scaled difference squared (z axis: feature)
+                d_K_d_logl = (X_l[:, nax, :] - X_l[nax, :, :])**2
                 # pairwise scaled distance (summing over feature)
                 R = np.sqrt(d_K_d_logl.sum(axis=2))
                 # deal with singularity
@@ -873,9 +873,9 @@ class MaternKernel(StationaryMixin, Kernel):
                 K = J * f(B)
                 # jacobian
                 if d == 1:
-                    d_K_d_logl *= (J * RR * sqrt(d))[:, :, newaxis]
+                    d_K_d_logl *= (J * RR * sqrt(d))[:, :, nax]
                 else:
-                    d_K_d_logl *= (J * RR * sqrt(d) * g(B))[:, :, newaxis]
+                    d_K_d_logl *= (J * RR * sqrt(d) * g(B))[:, :, nax]
                 return K, d_K_d_logl
         return fun
 
@@ -900,10 +900,9 @@ class PeriodicKernel(StationaryMixin, Kernel):
     """
     periodic kernel (RBF kernel in u-space)
 
-    k(x,z) = exp( -2 * || sin( π/p * (x-z)) / l ||**2 )
+    k(x,z) = exp( -2 * || sin( π/p * (x-z) ) / l ||**2 )
 
     p (period): positive float (isotropic) or positive array (anisotropic)
-                where periods define contours' shape and scale
     l (length scale): positive float (isotropic) or positive array (anisotropic)
     """
 
@@ -969,8 +968,9 @@ class PeriodicKernel(StationaryMixin, Kernel):
                 p = exp(log_params[0])
                 l = exp(log_params[1])
                 # map to u-space
-                X_p = X * (pi/p)
-                D_p = X_p[:, newaxis, :] - X_p[newaxis, :, :]
+                X_p = X * (pi / p)
+                # pairwise perioded difference squared (z axis: feature)
+                D_p = X_p[:, nax, :] - X_p[nax, :, :]
                 sinDp = np.sin(D_p)
                 # pairwise scaled l2 distance squared
                 R2_l2 = ((2./l * sinDp) ** 2).sum(axis=2)
@@ -993,8 +993,9 @@ class PeriodicKernel(StationaryMixin, Kernel):
                 p = np.exp(log_params[:dim])
                 l = np.exp(log_params[-dim:])
                 # map to u-space
-                X_p = np.einsum('ij,j->ij', X, pi/p)
-                D_p = X_p[:, newaxis, :] - X_p[newaxis, :, :]
+                X_p = np.einsum('ij,j->ij', X, pi / p)
+                # pairwise perioded difference squared (z axis: feature)
+                D_p = X_p[:, nax, :] - X_p[nax, :, :]
                 sinDp = np.sin(D_p)
                 sin2Dp4_l2 = np.einsum('ijk,k->ijk', sinDp, 2./l) ** 2
                 # pairwise scaled l2 distance squared
@@ -1002,9 +1003,9 @@ class PeriodicKernel(StationaryMixin, Kernel):
                 # kernel
                 K = np.exp(R2_l2 / -2.)
                 # jacobian
-                d_K_d_logp = K[:, :, newaxis] * D_p * np.cos(D_p) * sinDp
+                d_K_d_logp = K[:, :, nax] * D_p * np.cos(D_p) * sinDp
                 d_K_d_logp = np.einsum('ijk,k->ijk', d_K_d_logp, 4./l**2)
-                d_K_d_logl = K[:, :, newaxis] * sin2Dp4_l2
+                d_K_d_logl = K[:, :, nax] * sin2Dp4_l2
                 return K, np.dstack((d_K_d_logp, d_K_d_logl))
         return fun
 
@@ -1020,44 +1021,32 @@ class PeriodicKernel(StationaryMixin, Kernel):
                                   'number of periods / length scales.'  )
             l = self.l
             p = self.p
-        X_p = np.einsum('ij,j->ij', X, pi/p)
-        Z_p = np.einsum('ij,j->ij', Z, pi/p)
-        sinDp = np.sin(X_p[:, newaxis, :] - Z_p[newaxis, :, :])
+        X_p = np.einsum('ij,j->ij', X, pi / p)
+        Z_p = np.einsum('ij,j->ij', Z, pi / p)
+        sinDp = np.sin(X_p[:, nax, :] - Z_p[nax, :, :])
         R2_l2 = (np.einsum('ijk,k->ijk', sinDp, 2./l) ** 2).sum(axis=2)
         K = np.exp(R2_l2 / -2.)
         return K
 
-# FIXME: add CosineKernel, spectral kernel = RBFKernel * CosineKernel?
 
-class SpectralKernel(StationaryMixin, Kernel):
+class CosineKernel(StationaryMixin, Kernel):
     """
-    additive component of spectral mixture kernel
-    whose spectral densitiy is Gaussian(p, l)
+    cosine kernel (as l -> inf, zero-mean periodic(l) -> cosine)
 
-    k(x,z) = exp( - 1/2 * || (x-z)/l ||**2 ) * cos(p.T @ (x-z))
+    k(x,z) = Π_i cos( 2π/p * (x_i-z_i) )
 
     p (period): positive float (isotropic) or positive array (anisotropic)
-                defines contours' shape and scale
-    l (length scale): positive float (isotropic) or positive array (anisotropic)
     """
 
-    def __init__(self, p: float = 1.0, l: float = 1.0,
-                 p_bounds: B = (1e-4, 1e+4), l_bounds: B = (1e-4, 1e+4)):
+    def __init__(self, p: float = 1.0, p_bounds: B = (1e-4, 1e+4)):
         check_positive_scalar_array(p, 'period')
-        check_positive_scalar_array(l, 'length scale')
-        if not (type(p) == type(l) == float or
-                isinstance(p, ndarray) and isinstance(l, ndarray) and
-                len(p) == len(l)):
-            raise ValueError('periods and length scales must be of same size.')
-        self._thetas = Thetas.from_seq((p, l), (p_bounds, l_bounds), log)
+        self._thetas = Thetas.from_seq((p,), (p_bounds,), log)
 
     def __repr__(self):
         return self.__str__()
 
     def __str__(self):
-        p = formatter(self.p)
-        l = formatter(self.l)
-        return 'SpectralKernel(p={}, l={})'.format(p, l)
+        return 'CosineKernel(p={})'.format(formatter(self.p))
 
     @property
     def thetas(self):
@@ -1065,7 +1054,122 @@ class SpectralKernel(StationaryMixin, Kernel):
 
     @property
     def hyperparameters(self):
-        return {'period': self.p, 'length scale': self.l}
+        return {'period': self.p}
+
+    @property
+    def isotropic(self):
+        if len(self.thetas) == 1:
+            return True
+        else:
+            return False
+
+    @property
+    def p(self):
+        if self.isotropic:
+            return exp(self.thetas.values)
+        else:
+            return np.exp(self.thetas.values)
+
+    def _set(self, log_params: ndarray):
+        self._thetas.set(log_params)
+
+    def _obj(self, X: ndarray) -> Callable:
+        k = X.shape[1]  # number of features
+        masks = np.full((k, k), True, dtype=bool)
+        masks[np.diag_indices_from(masks)] = False
+        if self.isotropic:
+            def fun(log_params: ndarray) -> Tuple[ndarray, ndarray]:
+                assert is_array(log_params, 1, np.number)
+                # period
+                p = exp(log_params[0])
+                # map to u-space
+                X_p = X * (2 * pi / p)
+                # pairwise perioded difference squared (z axis: feature)
+                D_p = X_p[:, nax, :] - X_p[nax, :, :]
+                cosDp = np.cos(D_p)
+                # kernel
+                K = cosDp.prod(axis=2)
+                # jacobian
+                d_K_d_logp = np.sin(D_p) * D_p
+                for kk in range(k):
+                    d_K_d_logp[:,:,kk] *= cosDp[:,:,masks[kk]].prod(axis=2)
+                return K, d_K_d_logp.sum(axis=2, keepdims=True)
+        else:
+            if len(self.p) != X.shape[1]:
+                raise ValueError( 'number of features must agree '
+                                  'with number of length scales.'  )
+            def fun(log_params: ndarray) -> Tuple[ndarray, ndarray]:
+                assert is_array(log_params, 1, np.number)
+                # period
+                p = np.exp(log_params)
+                # map to u-space
+                X_p = np.einsum('ij,j->ij', X, 2 * pi / p)
+                # pairwise perioded difference squared (z axis: feature)
+                D_p = X_p[:, nax, :] - X_p[nax, :, :]
+                cosDp = np.cos(D_p)
+                # kernel
+                K = cosDp.prod(axis=2)
+                # jacobian
+                d_K_d_logp = np.sin(D_p) * D_p
+                for kk in range(k):
+                    d_K_d_logp[:,:,kk] *= cosDp[:,:,masks[kk]].prod(axis=2)
+                return K, d_K_d_logp
+        return fun
+
+    def __call__(self, X: ndarray, Z: ndarray) -> ndarray:
+        super().__call__(X, Z)
+        k = X.shape[1]  # number of features
+        if self.isotropic:
+            p = self.p * np.ones((k,))
+        else:
+            if len(self.p) != k:
+                raise ValueError( 'number of features must agree '
+                                  'with number of periods.'  )
+            p = self.p
+        X_p = np.einsum('ij,j->ij', X, 2 * pi / p)
+        Z_p = np.einsum('ij,j->ij', Z, 2 * pi / p)
+        K = np.cos(X_p[:, nax, :] - Z_p[nax, :, :]).prod(axis=2)
+        return K
+
+
+class SpectralKernel(StationaryMixin, Kernel):
+    """
+    spectral kernel, with spectral density (independent) Gaussian(u, v**2)
+
+    k_q(x,z) = cos( 2π * u.T @ (x-z) ) * exp( -2π**2 * || (x-z) * v ||**2 )
+
+    u (mean): positive float (isotropic) or positive array (anisotropic)
+    v (sd): positive float (isotropic) or positive array (anisotropic)
+
+    model spectral density with scale-location mixture of Gaussians =>
+    spectral mixture kernel, k_sm = Σ_q k_q
+    """
+
+    def __init__(self, u: float = 1.0, v: float = 1.0,
+                 u_bounds: B = (1e-4, 1e+4), v_bounds: B = (1e-4, 1e+4)):
+        check_positive_scalar_array(u, 'mean')
+        check_positive_scalar_array(v, 'sd')
+        if not (type(u) == type(v) == float or
+                isinstance(u, ndarray) and isinstance(v, ndarray) and
+                len(u) == len(v)):
+            raise ValueError('means and sds must be of same size.')
+        self._thetas = Thetas.from_seq((u, v), (u_bounds, v_bounds), log)
+
+    def __repr__(self):
+        return self.__str__()
+
+    def __str__(self):
+        u = formatter(self.u)
+        v = formatter(self.v)
+        return 'SpectralKernel(u={}, v={})'.format(u, v)
+
+    @property
+    def thetas(self):
+        return self._thetas
+
+    @property
+    def hyperparameters(self):
+        return {'mean': self.u, 'sd': self.v}
 
     @property
     def isotropic(self):
@@ -1079,14 +1183,14 @@ class SpectralKernel(StationaryMixin, Kernel):
         return len(self.thetas) // 2
 
     @property
-    def p(self):
+    def u(self):
         if self.isotropic:
             return exp(self.thetas.values[0])
         else:
             return np.exp(self.thetas.values[:self.dim])
 
     @property
-    def l(self):
+    def v(self):
         if self.isotropic:
             return exp(self.thetas.values[-1])
         else:
@@ -1101,72 +1205,65 @@ class SpectralKernel(StationaryMixin, Kernel):
             def fun(log_params: ndarray) -> Tuple[ndarray, ndarray]:
                 assert is_array(log_params, 1, np.number)
                 # period and length scale
-                p = exp(log_params[0])
-                l = exp(log_params[1])
+                u = exp(log_params[0])
+                v = exp(log_params[1])
                 # pairwise scaled l2 distance squared
-                R2_l2 = R2 / l**2
-                # pairwise difference summed
-                D = (X[:, newaxis, :] - X[newaxis, :, :]).sum(axis=2)
+                R2v2 = R2 * v**2
                 # precompute
-                Dp = D * p
-                rbf = np.exp(R2_l2 / -2.)
+                Du_ = 2.*pi * u * (X[:, nax, :] - X[nax, :, :]).sum(axis=2)
+                rbf = np.exp(-2.*pi**2 * R2v2)
                 # kernel
-                K = rbf * np.cos(Dp)
+                K = rbf * np.cos(Du_)
                 # jacobian
-                d_K_d_logp = rbf * -np.sin(Dp) * Dp
-                d_K_d_logl = K * R2_l2
-                return K, np.dstack((d_K_d_logp, d_K_d_logl))
+                d_K_d_logu = rbf * np.sin(Du_) * -Du_
+                d_K_d_logv = K * R2v2 * (-4.*pi**2)
+                return K, np.dstack((d_K_d_logu, d_K_d_logv))
         else:
-            if len(self.l) != X.shape[1]:
+            if len(self.v) != X.shape[1]:
                 raise ValueError( 'number of features must agree '
-                                  'with number of length scales.'  )
+                                  'with number of means / sds.'  )
             dim = self.dim
             def fun(log_params: ndarray) -> Tuple[ndarray, ndarray]:
                 assert is_array(log_params, 1, np.number)
                 # period and length scale
-                p = np.exp(log_params[:dim])
-                l = np.exp(log_params[-dim:])
-                X_l = np.einsum('ij,j->ij', X, 1./l)
-                # pairwise scaled difference squared (feature being 3rd axis)
-                d_K_d_logl = (X_l[:, newaxis, :] - X_l[newaxis, :, :])**2
+                u = np.exp(log_params[:dim])
+                v = np.exp(log_params[-dim:])
+                Xv = np.einsum('ij,j->ij', X, v)
+                # pairwise scaled difference squared (z axis: feature)
+                d_K_d_logv = (Xv[:, nax, :] - Xv[nax, :, :])**2
                 # pairwise scaled l2 distance squared (summing over feature)
-                R2_l2 = d_K_d_logl.sum(axis=2)
-                # pairwise difference
-                D = X[:, newaxis, :] - X[newaxis, :, :]
+                R2v2 = d_K_d_logv.sum(axis=2)
                 # pairwise perioded difference
-                d_K_d_logp = D * p[newaxis, newaxis, :]
+                d_K_d_logu = (X[:, nax, :] - X[nax, :, :]) * u[nax, nax, :]
                 # precompute
-                Dp = d_K_d_logp.sum(axis=2)
-                rbf = np.exp(R2_l2 / -2.)
+                Du_ = 2.*pi * d_K_d_logu.sum(axis=2)
+                rbf = np.exp(-2.*pi**2 * R2v2)
                 # kernel
-                K = rbf * np.cos(Dp)
+                K = rbf * np.cos(Du_)
                 # jacobian
-                d_K_d_logp *= (rbf * -np.sin(Dp))[:, :, newaxis]
-                d_K_d_logl *= K[:, :, newaxis]
-                return K, np.dstack((d_K_d_logp, d_K_d_logl))
+                d_K_d_logu *= (-2.*pi * rbf * np.sin(Du_))[:, :, nax]
+                d_K_d_logv *= (-4.*pi**2 * K)[:, :, nax]
+                return K, np.dstack((d_K_d_logu, d_K_d_logv))
         return fun
 
     def __call__(self, X: ndarray, Z: ndarray) -> ndarray:
         super().__call__(X, Z)
         k = X.shape[1]  # number of features
         if self.isotropic:
-            l = self.l * np.ones((k,))
-            p = self.p * np.ones((k,))
+            u = self.u * np.ones((k,))
+            v = self.v * np.ones((k,))
         else:
-            if len(self.l) != k:
+            if len(self.u) != k:
                 raise ValueError( 'number of features must agree '
-                                  'with number of length scales.'  )
-            if len(self.p) != k:
-                raise ValueError( 'number of features must agree '
-                                  'with number of length scales.'  )
-            l = self.l
-            p = self.p
-        X_l = np.einsum('ij,j->ij', X, 1./l)
-        Z_l = np.einsum('ij,j->ij', Z, 1./l)
-        R2_l2 = dist(X_l, Z_l, metric='sqeuclidean')
-        Rp = ((X[:, newaxis, :] - Z[newaxis, :, :]) * \
-               p[newaxis, newaxis, :]).sum(axis=2)
-        K = np.exp(R2_l2 / -2.) * np.cos(Rp)
+                                  'with number of means / sds.'  )
+            u = self.u
+            v = self.v
+        Xv = np.einsum('ij,j->ij', X, v)
+        Zv = np.einsum('ij,j->ij', Z, v)
+        R2v2 = dist(Xv, Zv, metric='sqeuclidean')
+        Du = ((X[:, nax, :] - Z[nax, :, :]) * u[nax, nax, :])\
+             .sum(axis=2)
+        K = np.cos(2*pi * Du) * np.exp(-2.*pi**2 * R2v2)
         return K
 
 
@@ -1175,14 +1272,17 @@ class LinearKernel(NonStationaryMixin, Kernel):
     linear kernel
 
     k(x,z) = (x/l).T @ (z/l)
-    # FIXME: reparametrise ((x-c)/l).T @ ((z-c)/l)?
+
     l (length scale): positive float (isotropic) or positive array (anisotropic)
+
+    ..todo:: reparametrise to k(x,z) = ((x-c)/l).T @ ((z-c)/l)
     """
 
     def __init__(self, l: V = 1.0, l_bounds: B = (1e-4, 1e+4)):
         super().__init__()
         check_positive_scalar_array(l, 'length scale')
         self._thetas = Thetas.from_seq((l,), (l_bounds,), log)
+        # mimic spectral kernel's structure
 
     def __repr__(self):
         return self.__str__()
@@ -1227,7 +1327,7 @@ class LinearKernel(NonStationaryMixin, Kernel):
                 # kernel
                 K = 1. / l**2 * X2
                 # jacobian
-                d_K_d_logl = (K * -2.)[:, :, newaxis]
+                d_K_d_logl = (K * -2.)[:, :, nax]
                 return K, d_K_d_logl
         else:
             if len(self.l) != X.shape[1]:
@@ -1238,7 +1338,7 @@ class LinearKernel(NonStationaryMixin, Kernel):
                 # length scale
                 l = np.exp(log_params)
                 X_l = np.einsum('ij,j->ij', X, 1./l)
-                # pairwise product (feature being 3rd axis)
+                # pairwise product (z axis: feature)
                 d_K_d_logl = np.einsum('ik,jk->ijk', X_l, X_l)
                 # kernel
                 K = d_K_d_logl.sum(axis=2)
@@ -1267,7 +1367,7 @@ class NeuralKernel(NonStationaryMixin, Kernel):
     """
     neural kernel
 
-    k(x, z) = 2/pi * arcsin( (x/l).T @ (z/l) + c ) /
+    k(x, z) = 2/π * arcsin( (x/l).T @ (z/l) + c ) /
               sqrt((||x/l||**2 + c + 1) * (||z/l||**2 + c + 1)) )
 
     c (intercept deviation): positive float
@@ -1340,10 +1440,10 @@ class NeuralKernel(NonStationaryMixin, Kernel):
                 D = N2_lc2 * np.sqrt(1 - J**2)  # denominator of jacobians
                 # jacobian
                 d_K_d_logc = 2.*c / pi / D * \
-                            (N_lc - .5*J*(n_lc[:, newaxis]+n_lc[newaxis, :]))
+                            (N_lc - .5*J*(n_lc[:, nax]+n_lc[nax, :]))
                 d_K_d_logl = 2./pi / D * (N_lc * X2 * (-2./ l**2) +
-                             J * ((n[:, newaxis]*n[newaxis, :])*(2./l**4)   +
-                                  (n[:, newaxis]+n[newaxis, :])*((c+1)/l**2) ) )
+                             J * ((n[:, nax]*n[nax, :])*(2./l**4)   +
+                                  (n[:, nax]+n[nax, :])*((c+1)/l**2) ) )
                 return K, np.dstack((d_K_d_logc, d_K_d_logl))
         else:
             def fun(log_params: ndarray) -> Tuple[ndarray, ndarray]:
@@ -1353,7 +1453,7 @@ class NeuralKernel(NonStationaryMixin, Kernel):
                 l = np.exp(log_params[1:])
                 # precompute
                 X_l = np.einsum('ij,j->ij', X, 1./l)
-                # scaled pairwise Hadamard product (feature being 3rd axis)
+                # scaled pairwise Hadamard product (z axis: feature)
                 W = np.einsum('ik,jk->ijk', X_l, X_l)
                 # scaled pairwise dot product
                 X2_l2 = W.sum(axis=2)
@@ -1369,10 +1469,9 @@ class NeuralKernel(NonStationaryMixin, Kernel):
                 D = N2_lc2 * np.sqrt(1 - J**2)  # denominator of jacobians
                 # jacobian
                 d_K_d_logc = 2.*c / pi / D * \
-                            (N_lc - .5*J * (n_lc[:, newaxis]+n_lc[newaxis, :]))
-                d_K_d_logl = 2./pi / D[:, :, newaxis] * \
-                             ( (-2.*N_lc)[:, :, newaxis] * W +
-                               J[:, :, newaxis] *
+                             (N_lc - .5*J * (n_lc[:, nax] + n_lc[nax, :]))
+                d_K_d_logl = 2./pi / D[:, :, nax] * \
+                             ( (-2.*N_lc)[:, :, nax] * W + J[:, :, nax] *
                                (np.einsum('i,jjk->ijk', n_lc, W) +
                                 np.einsum('j,iik->ijk', n_lc, W)   ) )
                 return K, np.dstack((d_K_d_logc, d_K_d_logl))
@@ -1390,8 +1489,8 @@ class NeuralKernel(NonStationaryMixin, Kernel):
             l = self.l
         X_l = np.einsum('ij,j->ij', X, 1./l)
         Z_l = np.einsum('ij,j->ij', Z, 1./l)
-        K = ((X_l**2).sum(axis=1) + self.c + 1.)[:, newaxis] * \
-            ((Z_l**2).sum(axis=1) + self.c + 1.)[newaxis, :]
+        K = ((X_l**2).sum(axis=1) + self.c + 1.)[:, nax] * \
+            ((Z_l**2).sum(axis=1) + self.c + 1.)[nax, :]
         K = (np.einsum('ik,jk->ij', X_l, Z_l) + self.c) / np.sqrt(K)
         K = 2./pi * np.arcsin(K)
         return K
